package net.npaka.serviceex;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.os.RemoteException;
import android.widget.Toast;

//サービスの定義(7)
public class MyService extends Service {
    private Handler handler = new Handler();
    private boolean running = false;
    private String  message = "Message";

    //サービス生成時に呼ばれる
    @Override
    public void onCreate() {
        super.onCreate();
    }
    
    //サービス開始時に呼ばれる
    public int onStartCommand(Intent intent, int flags, int startId) {
        super.onStartCommand(intent, flags, startId);
        
        //ノーティフィケーションの表示
        showNotification(this, R.drawable.ic_launcher,
            "自作サービス",
            "自作サービスを操作します",
            "情報");

        //サービスの開始
        Thread thread = new Thread(){public void run() {
            running = true;
            while (running) {
                handler.post(new Runnable(){public void run() {
                    toast(MyService.this, message);
                }});
                try {
                    Thread.sleep(3000);
                } catch (Exception e) {
                }
            }
        }};
        thread.start();
        return START_NOT_STICKY;
    }
    
    //サービス停止時に呼ばれる
    @Override
    public void onDestroy() {
        running = false;
        super.onDestroy();
    }
    
    //サービス接続時に呼ばれる
    @Override
    public IBinder onBind(Intent intent) {
        return IMyServiceBinder;
    }
    
    //ノーティフィケーションの表示
    public static void showNotification(Context context,
        int iconId, String title, String text, String info) {
        //ノーティフィケーションオブジェクトの生成(9)
        Notification.Builder builder = new Notification.Builder(context);
        builder.setWhen(System.currentTimeMillis());
        builder.setContentTitle(title);
        builder.setContentText(text);
        builder.setContentInfo(info);
        builder.setSmallIcon(iconId);
        
        //ペンディングインテントの指定(10)
        Intent intent = new Intent(context, ServiceEx.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        builder.setContentIntent(PendingIntent.getActivity(
            context, 0, intent, PendingIntent.FLAG_ONE_SHOT));
        
        //ノーティフィケーションの表示(11)
        NotificationManager nm = (NotificationManager)
            context.getSystemService(Context.NOTIFICATION_SERVICE);
        nm.cancel(0);
        nm.notify(0, builder.getNotification());
    }

    //トーストの表示　
    private static void toast(Context context, String text) {
        Toast.makeText(context, text, Toast.LENGTH_SHORT).show();
    }
    
    //バインダの生成(8)
    private final IMyService.Stub IMyServiceBinder = new IMyService.Stub() {
        public void setMessage(String msg) throws RemoteException {
            message = msg;
        }
    };
}