package net.npaka.gcmex;

import android.app.IntentService;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;

import com.google.android.gms.gcm.GoogleCloudMessaging;

//GCMサーバからの通知受信時の処理を行うサービス(4)
public class GCMIntentService extends IntentService {

    //コンストラクタ
    public GCMIntentService() {
        super(GCMIntentService.class.getName());
    }

    //コンストラクタ
    public GCMIntentService(String name) {
        super(name);
    }

    //処理時に呼ばれる
    @Override
    protected void onHandleIntent(Intent intent) {
        Bundle extras = intent.getExtras();
        Context context = getApplicationContext();

        //GCMのメッセージ種別の取得
        GoogleCloudMessaging gcm = GoogleCloudMessaging.getInstance(this);
        String messageType = gcm.getMessageType(intent);

        if (!extras.isEmpty()) {
            //GCMサーバからのエラー通知
            if (GoogleCloudMessaging.MESSAGE_TYPE_SEND_ERROR.equals(messageType)) {
                String message = extras.toString();
                GCMEx.showMessage(context, message);
                showNotification(context, R.drawable.ic_launcher,
                    "GCMサーバからのエラー通知", message, "");
            }
            //GCMサーバからのメッセージ削除通知
            else if (GoogleCloudMessaging.MESSAGE_TYPE_DELETED.equals(messageType)) {
                String message = extras.toString();
                GCMEx.showMessage(context, message);
                showNotification(context, R.drawable.ic_launcher,
                    "GCMサーバからのメッセージ削除通知", message, "");
            }
            //GCMサーバからのメッセージ受信
            else if (GoogleCloudMessaging.MESSAGE_TYPE_MESSAGE.equals(messageType)) {
                String message = extras.getString("message");
                GCMEx.showMessage(context, message);
                showNotification(context, R.drawable.ic_launcher,
                    "GCMサーバからのメッセージ受信", message, "");
            }
        }

        //サービスの処理の終了をブロードキャストレシーバーに通知(3)
        GCMBroadcastReceiver.completeWakefulIntent(intent);
    }

    //ノーティフィケーションの表示
    private static void showNotification(Context context,
        int iconId, String title, String text, String info) {
        Notification.Builder builder=new Notification.Builder(context);
        builder.setWhen(System.currentTimeMillis());
        builder.setContentTitle(title);
        builder.setContentText(text);
        builder.setContentInfo(info);
        builder.setSmallIcon(iconId);
        Intent intent=new Intent(context,GCMEx.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        builder.setContentIntent(PendingIntent.getActivity(
            context,0,intent, PendingIntent.FLAG_ONE_SHOT));
        NotificationManager nm=(NotificationManager)
            context.getSystemService(Context.NOTIFICATION_SERVICE);
        nm.cancel(0);
        nm.notify(0,builder.getNotification());
    }
}
