package net.npaka.wifidirectex;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.wifi.WifiManager;
import android.net.wifi.WpsInfo;
import android.net.wifi.p2p.WifiP2pConfig;
import android.net.wifi.p2p.WifiP2pDevice;
import android.net.wifi.p2p.WifiP2pDeviceList;
import android.net.wifi.p2p.WifiP2pInfo;
import android.net.wifi.p2p.WifiP2pManager;
import android.net.wifi.p2p.WifiP2pManager.Channel;
import android.net.wifi.p2p.WifiP2pManager.ConnectionInfoListener;
import android.net.wifi.p2p.WifiP2pManager.PeerListListener;
import android.os.Bundle;
import android.os.Handler;
import android.provider.Settings;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

//Wi-Fi Direct
public class WiFiDirectEx extends Activity implements 
    View.OnClickListener, PeerListListener, ConnectionInfoListener {
    private final static String BR = System.getProperty("line.separator");
    private final static int WC = LinearLayout.LayoutParams.WRAP_CONTENT;
    private final static int MP = LinearLayout.LayoutParams.MATCH_PARENT;

    //メッセージ定数
    public static final int MSG_STATE_CHANGE = 1;
    public static final int MSG_READ         = 2;
    
    //リクエスト定数
    private static final int RQ_CONNECT_DEVICE = 1;
    
    //Wi-Fi Direct
    private WifiP2pManager      manager;    //Wi-Fi P2Pマネージャ
    private Channel             channel;    //チャンネル
    private BroadcastReceiver   receiver;   //ブロードキャストレシーバー
    private ChatManager         chatManager;//チャットマネージャ
    private List<WifiP2pDevice> devices = new ArrayList<WifiP2pDevice>();
    private Handler             handler = new Handler();
    
    //UI
    private EditText edtSend;   //送信エディットテキスト
    private Button   btnSend;   //送信ボタン
    private TextView lblReceive;//受信ラベル
    
    public static WiFiDirectEx current;
    
    //アクティビティ起動時に呼ばれる
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        current = this;
        
        //レイアウトの生成
        LinearLayout layout = new LinearLayout(this);
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout); 
        setTitle("WiFiDirectEx - 未接続");
        
        //送信エディットテキストの生成
        edtSend = new EditText(this);
        edtSend.setText("");
        edtSend.setLayoutParams(new LinearLayout.LayoutParams(MP, WC));
        layout.addView(edtSend);
        
        //送信ボタンの生成
        btnSend = new Button(this);
        btnSend.setText("送信");
        btnSend.setOnClickListener(this);
        btnSend.setLayoutParams(new LinearLayout.LayoutParams(WC, WC));
        layout.addView(btnSend);

        //受信ラベルの生成
        lblReceive = new TextView(this);
        lblReceive.setText("");
        lblReceive.setTextSize(16.0f);
        lblReceive.setLayoutParams(new LinearLayout.LayoutParams(MP, WC));
        layout.addView(lblReceive);   
        
        //Wi-Fi P2Pマネージャの初期化(1)
        manager = (WifiP2pManager)getSystemService(Context.WIFI_P2P_SERVICE);
        channel = manager.initialize(this, getMainLooper(), null);
        
        //チャットマネージャ
        chatManager = new ChatManager(this);
    }
    
    //アクティビティスタート時に呼ばれる
    @Override
    public void onStart() {
        super.onStart();
        WifiManager wifiManager = (WifiManager) getSystemService(WIFI_SERVICE);
        wifiManager.setWifiEnabled(true);        
        
        //Wi-Fi Directのブロードキャストレシーバーの登録(2)
        receiver = new WiFiDirectReceiver(this, manager, channel);
        IntentFilter filter = new IntentFilter();
        filter.addAction(WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION);
        filter.addAction(WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION);
        filter.addAction(WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION);
        filter.addAction(WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION);
        registerReceiver(receiver, filter);
        
        //サーバの接続待ちスレッドの開始
        if (chatManager.getState() == ChatManager.STATE_NONE) {
            chatManager.accept();
        }
    }
    
    //アクティビティ停止時に呼ばれる
    @Override
    public void onStop() {
        super.onStop();

        //サーバの接続待ちスレッドの停止
        chatManager.stop();
        
        //端末との切断
        disconnectDevice();
        
        //Wi-Fi Directのブロードキャストレシーバーの解除(2)
        unregisterReceiver(receiver);        
    }
    
    //Wi-Fi Direct端末一覧の更新時に呼ばれる(4)
    public void onPeersAvailable(WifiP2pDeviceList deviceList) {
        devices.clear();
        devices.addAll(deviceList.getDeviceList());
        if (DeviceListActivity.activity != null) {
            DeviceListActivity.activity.update(devices);
        }
    }
    
    //Wi-Fi Direct接続情報の通知時に呼ばれる(5)
    public void onConnectionInfoAvailable(WifiP2pInfo info) {
        addText("Wi-Fi Directの接続情報>isGroupOwner="+info.isGroupOwner);
        if (!info.isGroupOwner) {
            chatManager.connect(info.groupOwnerAddress.getHostAddress());
        }
    }
    
    //端末との接続(7)
    public void connectDevice(final WifiP2pDevice device) {
        //接続設定の生成
        WifiP2pConfig config = new WifiP2pConfig();
        config.deviceAddress = device.deviceAddress;
        config.wps.setup = WpsInfo.PBC;
        manager.connect(channel, config, null);
    }
    
    //端末との切断(8)
    public void disconnectDevice() {
        if (status == 0) {
            manager.removeGroup(channel, null);
        } else if (status == 1) {
            manager.cancelConnect(channel, null);
        }
    }
    
    public static int status=0;
    
    //オプションメニュー生成時に呼ばれる
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        MenuItem item0 = menu.add(0, 0, 0, "端末検索");
        item0.setIcon(android.R.drawable.ic_search_category_default);
        MenuItem item1 = menu.add(0, 1, 0, "通信設定");
        item1.setIcon(android.R.drawable.ic_menu_preferences);
        return true;
    }

    //オプションメニュー選択時に呼ばれる
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == 0) {
            //端末検索(6)
            Intent intent = new Intent(this, DeviceListActivity.class);
            startActivityForResult(intent, RQ_CONNECT_DEVICE);
            
            //ピアの検索を開始
            //manager.discoverPeers(channel, null);
            return true;
        } else if (item.getItemId() == 1) {
            //通信設定
            startActivity(new Intent(Settings.ACTION_WIRELESS_SETTINGS));
            return true;
        }
        return false;
    }    
    
    //アクティビティ復帰時に呼ばれる
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        //端末検索
        if (requestCode == RQ_CONNECT_DEVICE){
            if (resultCode == Activity.RESULT_OK) {
                //端末との接続
                int idx = data.getExtras().getInt("device_index");
                connectDevice(devices.get(idx));
            }
        }
    }
    
    //状態の指定　
    public void setState(final String state) {
        handler.post(new Runnable(){public void run() {
            setTitle("WiFiDirectEx - "+state);
        }});
    }
    
    //受信テキストの追加
    public void addText(final String text) {
        handler.post(new Runnable(){public void run() {
            lblReceive.setText(text+BR+lblReceive.getText());
        }});
    }
    
    //ボタンクリックイベントの処理
    public void onClick(View v) {
        try {
            //メッセージの送信
            String message = edtSend.getText().toString();
            if (message.length() <= 0) return;
            chatManager.write(message.getBytes());
            addText(message);
            edtSend.setText("");
        } catch (Exception e) {
        }           
    }  
}
