package net.npaka.wifidirectex;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.net.NetworkInfo;
import android.net.wifi.p2p.WifiP2pDevice;
import android.net.wifi.p2p.WifiP2pManager;
import android.net.wifi.p2p.WifiP2pManager.Channel;

//Wi-Fi Directのブロードキャストレシーバー
public class WiFiDirectReceiver extends BroadcastReceiver {
    private WiFiDirectEx   parent  ;//本体
    private WifiP2pManager manager;//Wi-Fiマネージャ
    private Channel        channel;//チャンネル

    //コンストラクタ
    public WiFiDirectReceiver(WiFiDirectEx parent,
        WifiP2pManager manager, Channel channel) {
        this.parent  = parent;
        this.manager = manager;
        this.channel = channel;
    }

    //受信時に呼ばれる
    @Override
    public void onReceive(Context context, Intent intent) {
        String action = intent.getAction();
        //端末本体のWi-Fi Directの有効・無効の変更通知(3)
        if (WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION.equals(action)) {
            int state = intent.getIntExtra(WifiP2pManager.EXTRA_WIFI_STATE, -1);
            parent.addText("Wi-Fi Directの有効・無効>"+
                (state == WifiP2pManager.WIFI_P2P_STATE_ENABLED));
        } 
        //Wi-Fi Direct端末一覧の変更通知(4)
        else if (WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION.equals(action)) {
            //発見したピアの情報の要求→onPeersAvailable()
            manager.requestPeers(channel, parent);
        } 
        //Wi-Fi Direct通信状態の変更通知(5)
        else if (WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION.equals(action)) {
            NetworkInfo networkInfo = (NetworkInfo)intent.
                getParcelableExtra(WifiP2pManager.EXTRA_NETWORK_INFO);
            
            //接続
            if (networkInfo.isConnected()) {
                //端末の接続情報の要求→onConnectionInfoAvailable()
                manager.requestConnectionInfo(channel, parent);
            }
            parent.addText("Wi-Fi Directの通信状態>"+networkInfo.isConnected());
        } 
        //端末本体の状態の変更通知(6)
        else if (WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION.equals(action)) {
            WifiP2pDevice device = (WifiP2pDevice)intent.getParcelableExtra(
                WifiP2pManager.EXTRA_WIFI_P2P_DEVICE);
            String[] STATUS = {
                "CONNECTED", "INVITED", "FAILED", "AVAILABLE", "UNAVAILABLE"};
            parent.addText("端末本体の状態>"+device.deviceName+","+STATUS[device.status]);
            
            WiFiDirectEx.status = device.status;
        }
    }
}
