package net.npaka.actiongame;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.Point;
import android.view.Display;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

import java.util.Random;

//アクションゲーム
public class ActionView extends SurfaceView 
    implements SurfaceHolder.Callback, Runnable {
    //シーン定数(1)
    private final static int 
        S_TITLE    = 0,//タイトル
        S_PLAY     = 1,//プレイ
        S_GAMEOVER = 2;//ゲームオーバー

    //画面サイズ定数
    private final static int
        W = 800,//画面幅
        H = 480;//画面高さ

    //システム
    private SurfaceHolder holder;        //サーフェイスホルダー
    private Graphics g;                  //グラフィック
    private Thread   thread;             //スレッド
    private Bitmap[] bmp = new Bitmap[4];//ビットマップ
    private int      init = S_TITLE;     //初期化(1)
    private int      scene = S_TITLE;    //シーン(1)
    private boolean  touchDown;          //タッチダウン
    private String   message;            //メッセージ
    private int      score;              //スコア

    //プレイヤー(4)
    private int     playerY; //Y座標
    private int     jumpPow; //ジャンプ力
    private boolean jumpAble;//ジャンプ可

    //ブロック(2)
    private int   blockNum = 10;             //数
    private int   blockDX;                   //X相対位置
    private int[] blockH = new int[blockNum];//高さ
    
    //コンストラクタ
    public ActionView(Activity activity) {
        super(activity);
        
        //ビットマップの読み込み
        for (int i = 0; i < 4; i++) {
            bmp[i] = readBitmap(activity, "act"+i);
        }

        //サーフェイスホルダーの生成
        holder = getHolder();
        holder.addCallback(this);

        //画面サイズの指定
        Display display = activity.getWindowManager().getDefaultDisplay();
        Point p = new Point();
        display.getSize(p);
        int dw = H*p.x/p.y;

        //グラフィックスの生成
        g = new Graphics(dw, H, holder);
        g.setOrigin((dw-W)/2, 0);
    }
    
    //サーフェイス生成時に呼ばれる
    public void surfaceCreated(SurfaceHolder holder) {
        thread = new Thread(this);
        thread.start();
    }

    //サーフェイス終了時に呼ばれる
    public void surfaceDestroyed(SurfaceHolder holder) {
        thread = null;
    }

    //サーフェイス変更時に呼ばれる
    public void surfaceChanged(SurfaceHolder holder,
        int format, int w, int h) {
    }   
    
    //スレッドの処理
    public void run() {
        while(thread != null) {
            //初期化
            if (init >= 0) {
                scene = init;
                //タイトル
                if (scene == S_TITLE) {
                    message  = "Touch Start";
                    score    = 0;
                    playerY  = H-100;
                    jumpPow  = 0;
                    jumpAble = true;
                    blockDX  = 0;
                    for (int i = 0; i < 10; i++) blockH[i] = 1;
                } 
                //プレイ
                else if (scene == S_PLAY) {
                    message   = null;
                    touchDown = false;
                } 
                //ゲームオーバー
                else if (scene == S_GAMEOVER) {
                    message = "Game Over";
                }
                init = -1;
            }
            
            //プレイ時の処理
            if (scene == S_PLAY) {
                //スコア加算
                score++;

                //衝突判定
                if (playerY > H-blockH[2]*100) {
                    init = S_GAMEOVER;
                    jumpAble = false;
                }
                //上移動
                else if (jumpPow >= 0) {
                    playerY -= jumpPow*2;
                    jumpPow--;
                }
                //下移動(3)
                else {
                    playerY += 14;
                    jumpAble = false;
                    if (blockH[2] != 0 &&
                        playerY > H-blockH[2]*100) {
                        playerY = H-blockH[2]*100;
                        jumpAble = true;
                    }
                }
                //ブロックのスクロール(2)
                blockDX -= 10;
                if (blockDX == -100) {
                    blockDX = 0;

                    //横方向の位置を戻し、高さを右隣のブロックと同じに
                    System.arraycopy(blockH, 1, blockH, 0, blockNum-1);

                    //地面の高さを指新規ブロックの高さの設定(3)
                    int idx = rand(6);
                    //1つ前が穴のときは2つ前の高さ
                    if (blockH[blockNum-2] == 0) {
                        blockH[blockNum-1] = blockH[blockNum-3];
                    } 
                    //1/6の確率で穴
                    else if (idx == 0) {
                        blockH[blockNum-1] = 0;
                    } 
                    //1/6の確率で1段上
                    else if (idx == 1) {
                        blockH[blockNum-1] = blockH[blockNum-2]+1;
                        if (blockH[blockNum-1] > 4) blockH[blockNum-1] = 4;
                    } 
                    //1/6の確率で1段下
                    else if (idx == 2) {
                        blockH[blockNum-1] = blockH[blockNum-2]-1;
                        if (blockH[blockNum-1] < 1) blockH[blockNum-1] = 1;
                    }
                }

                //ジャンプのコントロール(4)
                if (jumpAble) {
                    if (touchDown) {
                        jumpAble = false;
                        jumpPow  = 18;
                    }
                } else {
                    if (!touchDown) {
                        jumpPow = -1;
                    }
                }
            }
            //ゲームオーバー時の処理
            else if (scene == S_GAMEOVER) {
                if (playerY < H+300) playerY += 20;
            }
            
            //背景の描画
            g.lock();
            g.drawBitmap(bmp[0], 0, 0);
            
            //プレイヤーの描画
            int idx = 3;
            if (jumpAble) idx = 2+(score/2)%2;
            g.drawBitmap(bmp[idx], 150, playerY-140);

            //ブロックの描画
            for (int i = 0; i < blockNum; i++) {
                g.drawBitmap(bmp[1], blockDX+i*100,
                    H-blockH[i]*100);
            }
            
            //スコアの描画
            g.setColor(Color.WHITE);
            g.setTextSize(30);
            g.drawText("SCORE "+num2str(score, 6),
                10+g.getOriginX(), 10-(int)g.getFontMetrics().ascent);
            
            //メッセージの描画
            if (message != null) {
                g.setTextSize(50);
                g.drawText(message,
                    (W-g.measureText(message))/2,
                    80-(int)g.getFontMetrics().ascent);
            }
            g.unlock();

            //スリープ
            try {
                Thread.sleep(30);
            } catch (Exception e) {
            }
        }
    }
    
    //タッチ時に呼ばれる
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        int touchAction = event.getAction();
        if (touchAction == MotionEvent.ACTION_DOWN) {
            if (scene == S_TITLE) {
                init = S_PLAY;
            } else if (scene == S_PLAY) {
                touchDown = true;
            } else {
                if (playerY >= H+300) init = S_TITLE;
            }
        } else if (touchAction == MotionEvent.ACTION_UP) {
            touchDown = false;
        }
        return true;
    }
    
    //乱数の取得
    private static Random rand = new Random();
    private static int rand(int num) {
        return (rand.nextInt()>>>1)%num;
    }
    
    //数値→文字列
    private static String num2str(int num, int len) {
        String str = ""+num;
        while(str.length() < len) str = "0"+str;
        return str;
    }
    
    //ビットマップの読み込み
    private static Bitmap readBitmap(Context context, String name) {
        int resID = context.getResources().getIdentifier(
            name, "drawable", context.getPackageName());
        return BitmapFactory.decodeResource(
            context.getResources(), resID);
    }
}