package net.npaka.puzzlegame;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Point;
import android.graphics.Rect;
import android.view.Display;
import android.view.MotionEvent;
import android.view.View;

import java.util.Random;

//パズルゲーム
public class PuzzleView extends View {
    //シーン定数(1)
    private final static int
        S_TITLE = 0,//タイトル
        S_PLAY  = 1,//プレイ
        S_CLEAR = 2;//クリア

    //画面サイズ定数
    private final static int
        W = 480,//画面幅
        H = 800;//画面高さ

    //システム
    private Graphics g;                  //グラフィックス
    private Rect     gSrc;               //グラフィックス描画元
    private Rect     gDst;               //グラフィックス描画先
    private int      scene = S_TITLE;    //シーン(1)
    private Bitmap[] bmp = new Bitmap[5];//ビットマップ
    private int[]    data = new int[25]; //ピース配列情報(3)
    private int      shuffle;            //シャッフル

    //コンストラクタ
    public PuzzleView(Activity activity) {
        super(activity);

        //ビットマップの読み込み
        for (int i = 0; i < 5; i++) {
            bmp[i] = readBitmap(activity, "pzl"+i);
        }

        //端末画面とゲーム画面のサイズの取得(2)
        Display display = activity.getWindowManager().getDefaultDisplay();
        Point p = new Point();
        display.getSize(p);
        int dh = W*p.y/p.x;

        //グラフィックスの生成(2)
        g = new Graphics(W, dh);
        g.setOrigin(0, (dh-H)/2);
        gSrc = new Rect(0, 0, W, dh);
        gDst = new Rect(0, 0, p.x, p.y);

        //初期化
        initScene(S_TITLE);
    }

    //シーンの初期化
    private void initScene(int scene) {
        this.scene = scene;

        //タイトル
        if (scene == S_TITLE) {
            for (int i = 0; i < 16; i++) data[i] = i;
        }
        //プレイ
        else if (scene == S_PLAY) {
            //シャッフルの実行(8)
            shuffle = 20;
            while (shuffle > 0) {
                if (movePiece(rand(4), rand(4))) shuffle--;
            }
        }
        invalidate();
    }

    //描画時に呼ばれる
    @Override
    protected void onDraw(Canvas canvas) {
        //背景の描画
        g.drawBitmap(bmp[0], 0, 0);

        //ピースの描画
        int px = (W-400)/2;
        int py = (H-400)/2;
        g.drawBitmap(bmp[2], px-2, py-2);
        for (int i = 0; i < 16; i++) {
            int sx = data[i]%4;
            int sy = data[i]/4;
            int dx = i%4;
            int dy = i/4;
            if (scene != S_PLAY || data[i] != 15) {
                g.drawBitmap(bmp[1],
                    new Rect(100*sx, 100*sy, 100*sx+100,100*sy+100),
                    new Rect(px+100*dx, py+100*dy, px+100*dx+100,
                    py+100*dy+100));
                g.drawBitmap(bmp[3], px+100*dx, py+100*dy);
            }
        }

        //メッセージの描画
        if (scene == S_TITLE || scene == S_CLEAR) {
            String str = (scene == S_TITLE) ? "15 Puzzle" : "Clear!";
            g.setColor(Color.WHITE);
            g.setTextSize(60);
            g.drawText(str, (W-g.measureText(str))/2,
                (200+g.getOriginY()-(int)g.getFontMetrics().top)/2);
        }

        //スタートボタンの描画
        if (scene == S_TITLE) {
            g.drawBitmap(bmp[4], 104,
                600+(200-g.getOriginY()-56)/2);
        }

        //画面に反映
        canvas.drawBitmap(g.getBitmap(), gSrc, gDst, null);
    }

    //タッチ時に呼ばれる
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        int touchX = (int)(event.getX()*W/getWidth());
        int touchY = (int)(event.getY()*H/getHeight());
        int touchAction = event.getAction();
        if (touchAction == MotionEvent.ACTION_DOWN) {
            //タイトル
            if (scene == S_TITLE) {
                initScene(S_PLAY);
            }
            //プレイ
            else if (scene == S_PLAY) {
                //タッチ位置からピースの列番号と行番号を求める(4)
                int px = (W-400)/2;
                int py = (H-400)/2;
                if (px < touchX && touchX < px+400 &&
                    py < touchY && touchY < py+400) {
                    int tx = (touchX-px)/100;
                    int ty = (touchY-py)/100;
                    movePiece(tx, ty);
                }
            }
            //ゲームオーバー
            else if (scene == S_CLEAR) {
                initScene(S_TITLE);
            }
        }
        return true;
    }

    //ピースの移動
    private boolean movePiece(int tx, int ty) {
        //空きマスの列番号と行番号を求める(5)
        int fx = 0;
        int fy = 0;
        for (int i = 0; i < 16; i++) {
            if (data[i] == 15) {
                fx = i%4;
                fy = i/4;
                break;
            }
        }
        if ((fx == tx && fy == ty) || (fx != tx && fy != ty)) {
            return false;
        }

        //ピースを上にスライド(6)
        if (fx == tx && fy < ty) {
            for (int i = fy; i < ty; i++) {
                data[fx+i*4] = data[fx+i*4+4];
            }
            data[tx+ty*4] = 15;
        }
        //ピースを下にスライド(6)
        else if (fx == tx && fy > ty) {
            for (int i = fy; i > ty; i--) {
                data[fx+i*4] = data[fx+i*4-4];
            }
            data[tx+ty*4] = 15;
        }
        //ピースを左にスライド(6)
        else if (fy == ty && fx < tx) {
            for (int i = fx; i < tx; i++) {
                data[i+fy*4] = data[i+fy*4+1];
            }
            data[tx+ty*4] = 15;
        }
        //ピースを右にスライド(6)
        else if (fy == ty && fx > tx) {
            for (int i = fx; i > tx; i--) {
                data[i+fy*4] = data[i+fy*4-1];
            }
            data[tx+ty*4] = 15;
        }

        //シャッフル時はクリアチェックは行わない
        if (shuffle > 0) {
            return true;
        }

        //ゲームのクリア判定(7)
        int clearCheck = 0;
        for (int i = 0; i < 16; i++) {
            if (data[i] == i) clearCheck++;
        }
        if (clearCheck == 16) {
            scene = S_CLEAR;
        }
        invalidate();
        return true;
    }

    //乱数の取得
    private static Random rand = new Random();
    private static int rand(int num) {
        return (rand.nextInt()>>>1)%num;
    }

    //数値→文字列
    private static String num2str(int num, int len) {
        String str = ""+num;
        while(str.length() < len) str = "0"+str;
        return str;
    }

    //ビットマップの読み込み
    private static Bitmap readBitmap(Context context, String name) {
        int resID = context.getResources().getIdentifier(
            name, "drawable", context.getPackageName());
        return BitmapFactory.decodeResource(
            context.getResources(), resID);
    }
}