package net.npaka.rpg;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.PixelFormat;
import android.graphics.Point;
import android.view.Display;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

import java.util.Random;

//RPGビュー
public class RPGView extends SurfaceView
    implements SurfaceHolder.Callback, Runnable {
    //シーン定数(1)
    private final static int
        S_START   = 0,//スタート
        S_MAP     = 1,//マップ
        S_APPEAR  = 2,//出現
        S_COMMAND = 3,//コマンド
        S_ATTACK  = 4,//攻撃
        S_DEFENCE = 5,//防御
        S_ESCAPE  = 6;//逃げる

    //画面サイズ定数
    private final static int
        W = 800,//画面幅
        H = 480;//画面高さ

    //キー定数
    private final static int
        KEY_NONE   = -1,//なし
        KEY_LEFT   = 0, //左
        KEY_RIGHT  = 1, //右
        KEY_UP     = 2, //上
        KEY_DOWN   = 3, //下
        KEY_1      = 4, //1
        KEY_2      = 5, //2
        KEY_SELECT = 6; //選択

    //マップ定数(2)
    private final static int[][] MAP = {
        {3, 3, 3, 3, 3, 3, 3, 3, 3, 3},
        {3, 1, 0, 0, 0, 0, 3, 0, 0, 3},
        {3, 0, 0, 0, 0, 0, 3, 3, 0, 3},
        {3, 0, 3, 3, 3, 3, 3, 3, 0, 3},
        {3, 0, 0, 3, 0, 0, 0, 3, 0, 3},
        {3, 3, 0, 3, 0, 3, 3, 3, 0, 3},
        {3, 0, 0, 3, 0, 0, 0, 0, 0, 3},
        {3, 0, 3, 3, 0, 3, 0, 3, 3, 3},
        {3, 0, 0, 0, 0, 3, 0, 0, 2, 3},
        {3, 3, 3, 3, 3, 3, 3, 3, 3, 3}};

    //勇者定数(3)
    private final static int[]
        YU_MAXHP   = {0, 30, 50, 70},//最大体力
        YU_ATTACK  = {0, 5, 10, 30}, //攻撃力
        YU_DEFENCE = {0, 0, 5, 10},  //守備力
        YU_EXP     = {0, 0, 3, 6};   //必要経験値

    //敵定数(4)
    private final static String[]
        EN_NAME = {"ザコ", "ボス"};//名前
    private final static int[]
        EN_MAXHP   = {10, 50},//最大体力
        EN_ATTACK  = {10, 26},//攻撃力
        EN_DEFENCE = {0, 16}, //守備力
        EN_EXP     = {1, 99}; //取得経験値

    //システム
    private SurfaceHolder holder;        //サーフェイスホルダー
    private Graphics g;                  //グラフィック
    private Thread   thread;             //スレッド
    private int      init = S_START;     //初期化(1)
    private int      scene;              //シーン(1)
    private int      key;                //キー
    private Bitmap[] bmp = new Bitmap[7];//ビットマップ

    //勇者パラメータ(3)
    private int yuX   = 1; //X座標
    private int yuY   = 2; //Y座標
    private int yuLV  = 1; //レベル
    private int yuHP  = 30;//体力
    private int yuEXP = 0; //経験値

    //敵パラメータ(4)
    private int enType;//種類
    private int enHP;  //体力

    //コンストラクタ
    public RPGView(Activity activity) {
        super(activity);

        //ビットマップの読み込み
        for (int i = 0; i < 7; i++) {
            bmp[i] = readBitmap(activity, "rpg"+i);
        }

        //サーフェイスホルダーの生成
        holder = getHolder();
        holder.addCallback(this);

        //画面サイズの指定
        Display display = activity.getWindowManager().getDefaultDisplay();
        Point p = new Point();
        display.getSize(p);
        int dw = H*p.x/p.y;

        //グラフィックスの生成
        g = new Graphics(dw, H, holder);
        g.setOrigin((dw-W)/2, 0);
    }

    //サーフェイスの生成
    public void surfaceCreated(SurfaceHolder holder) {
        thread = new Thread(this);
        thread.start();
    }

    //サーフェイスの終了
    public void surfaceDestroyed(SurfaceHolder holder) {
        thread = null;
    }

    //サーフェイスの変更
    public void surfaceChanged(SurfaceHolder holder,
        int format, int w, int h) {
    }

    //スレッドの処理
    public void run() {
        while(thread!=null) {
            //シーンの初期化
            if (init >= 0) {
                scene = init;
                //スタート
                if (scene == S_START) {
                    scene = S_MAP;
                    yuX   = 1;  //X座標
                    yuY   = 2;  //Y座標
                    yuLV  = 1;  //レベル
                    yuHP  = 30; //体力
                    yuEXP = 0;  //経験値
                }

                init = -1;
                key  = KEY_NONE;
            }

            //マップ
            if (scene == S_MAP) {
                //移動
                boolean flag=false;
                if (key == KEY_UP) {
                    if (MAP[yuY-1][yuX] <= 2) {yuY--;flag = true;}
                } else if (key == KEY_DOWN) {
                    if (MAP[yuY+1][yuX] <= 2) {yuY++;flag = true;}
                } else if (key == KEY_LEFT) {
                    if (MAP[yuY][yuX-1] <= 2) {yuX--;flag = true;}
                } else if (key == KEY_RIGHT) {
                    if (MAP[yuY][yuX+1] <= 2) {yuX++;flag = true;}
                }

                //敵出現の計算(5)
                if (flag) {
                    if (MAP[yuY][yuX] == 0 && rand(10) == 0) {
                        enType = 0;init = S_APPEAR;}
                    if (MAP[yuY][yuX] == 1) yuHP = YU_MAXHP[yuLV];
                    if (MAP[yuY][yuX] == 2) {enType = 1;init = S_APPEAR;}
                }

                //描画
                g.lock();
                for (int j = -3; j <= 3; j++) {
                    for (int i = -5;i <= 5; i++) {
                        int idx = 3;
                        if (0 <= yuX+i && yuX+i < MAP[0].length &&
                            0 <= yuY+j && yuY+j < MAP.length) {
                            idx = MAP[yuY+j][yuX+i];
                        }
                        g.drawBitmap(bmp[idx], W/2-40+80*i, H/2-40+80*j);
                    }
                }
                g.drawBitmap(bmp[4], W/2-40, H/2-40);
                drawStatus();
                g.unlock();
            }
            //出現の処理
            else if (scene == S_APPEAR) {
                //初期化
                enHP = EN_MAXHP[enType];

                //フラッシュ
                sleep(300);
                for (int i = 0; i < 6; i++) {
                    g.lock();
                    if (i%2 == 0) {
                        g.setColor(Color.rgb(0, 0, 0));
                    } else {
                        g.setColor(Color.rgb(255, 255, 255));
                    }
                    g.fillRect(0, 0, W, H);
                    g.unlock();
                    sleep(100);
                }

                //メッセージ
                drawBattle(EN_NAME[enType]+"があらわれた");
                waitSelect();
                init = S_COMMAND;
            }
            //コマンド
            else if (scene == S_COMMAND) {
                drawBattle("　　1.攻撃　　2.逃げる");
                key = KEY_NONE;
                while (init == -1) {
                    if (key == KEY_1) init = S_ATTACK;
                    if (key == KEY_2) init = S_ESCAPE;
                    sleep(100);
                }
            }
            //攻撃の処理
            else if (scene == S_ATTACK) {
                //メッセージ
                drawBattle("勇者の攻撃");
                waitSelect();

                //フラッシュ
                for (int i = 0; i < 10; i++) {
                    drawBattle("勇者の攻撃", i%2 == 0);
                    sleep(100);
                }

                //攻撃の計算(6)
                int damage = YU_ATTACK[yuLV]-EN_DEFENCE[enType]+rand(10);
                if (damage <= 1) damage = 1;
                if (damage >= 99) damage = 99;

                //メッセージ
                drawBattle(damage+"ダメージ与えた！");
                waitSelect();

                //体力の計算
                enHP -= damage;
                if (enHP <= 0) enHP = 0;

                //勝利
                init = S_DEFENCE;
                if (enHP == 0) {
                    //メッセージ
                    drawBattle(EN_NAME[enType]+"を倒した");
                    waitSelect();

                    //経験値計算
                    yuEXP += EN_EXP[enType];
                    if (yuLV<3 && YU_EXP[yuLV+1] <= yuEXP) {
                        yuLV++;
                        drawBattle("レベルアップした");
                        waitSelect();
                    }

                    //エンディング
                    if (enType == 1) {
                        g.lock();
                        g.setColor(Color.rgb(0, 0, 0));
                        g.fillRect(0, 0, W, H);
                        g.setColor(Color.rgb(255, 255, 255));
                        g.setTextSize(32);
                        String str = "Fin.";
                        g.drawText(str, (W-g.measureText(str))/2,
                            180-(int)g.getFontMetrics().top);
                        g.unlock();
                        waitSelect();
                        init = S_START;
                    }
                    init = S_MAP;
                }
            }
            //敵攻撃
            else if (scene == S_DEFENCE) {
                //メッセージ
                drawBattle(EN_NAME[enType]+"の攻撃");
                waitSelect();

                //フラッシュ
                for (int i = 0; i < 10; i++) {
                    if (i%2 == 0) {
                        g.lock();
                        g.setColor(Color.rgb(255, 255, 255));
                        g.fillRect(0, 0, W, H);
                        g.unlock();
                    } else {
                        drawBattle(EN_NAME[enType]+"の攻撃");
                    }
                    sleep(100);
                }

                //防御の計算(7)
                int damage = EN_ATTACK[enType]-YU_DEFENCE[yuLV]+rand(10);
                if (damage <= 1) damage = 1;
                if (damage >= 99) damage = 99;

                //メッセージ
                drawBattle(damage+"ダメージ受けた！");
                waitSelect();

                //体力の計算
                yuHP -= damage;
                if (yuHP <= 0) yuHP = 0;

                //敗北
                init = S_COMMAND;
                if (yuHP == 0) {
                    drawBattle("勇者は力尽きた");
                    waitSelect();
                    init = S_START;
                }
            }
            //逃げる
            else if (scene == S_ESCAPE) {
                //メッセージ
                drawBattle("勇者は逃げ出した");
                waitSelect();

                //逃げるの計算(8)
                init = S_MAP;
                if (enType == 1 || rand(100) <= 10) {
                    drawBattle(EN_NAME[enType]+"は回り込んだ");
                    waitSelect();
                    init = S_DEFENCE;
                }
            }
            //スリープ
            key = KEY_NONE;
            sleep(200);
        }
    }

    //戦闘画面の描画
    private void drawBattle(String message) {
        drawBattle(message, enHP >= 0);
    }

    //戦闘画面の描画
    private void drawBattle(String message,boolean visible) {
        int color = (yuHP != 0) ? Color.rgb(0, 0, 0) : Color.rgb(2555, 0, 0);
        g.lock();
        g.setColor(color);
        g.fillRect(0, 0, W, H);
        drawStatus();
        if (visible) {
            g.drawBitmap(bmp[5+enType],
                (W-bmp[5+enType].getWidth())/2,
                H-100-bmp[5+enType].getHeight());
        }
        g.setColor(Color.rgb(255, 255, 255));
        g.fillRect((W-504)/2, H-122, 504, 104);
        g.setColor(color);
        g.fillRect((W-500)/2, H-120, 500, 100);
        g.setColor(Color.rgb(255, 255, 255));
        g.setTextSize(32);
        g.drawText(message,
            (W-500)/2+50, 370-(int)g.getFontMetrics().top);
        g.unlock();
    }

    //ステータスの描画
    private void drawStatus() {
        int color = (yuHP != 0) ? Color.rgb(0, 0, 0) : Color.rgb(255, 0, 0);
        g.setColor(Color.rgb(255, 255, 255));
        g.fillRect((W-504)/2, 8, 504, 54);
        g.setColor(color);
        g.fillRect((W-500)/2, 10, 500, 50);
        g.setColor(Color.rgb(255, 255, 255));
        g.setTextSize(32);
        g.drawText("勇者　LV"+yuLV+"　HP"+yuHP+"/"+YU_MAXHP[yuLV],
            (W-500)/2+80, 15-(int)g.getFontMetrics().top);
    }

    //タッチ時に呼ばれる
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        int touchX = (int)(event.getX()*W/getWidth());
        int touchY = (int)(event.getY()*H/getHeight());
        int touchAction = event.getAction();
        if (touchAction == MotionEvent.ACTION_DOWN) {
            if (scene == S_MAP) {
                if (Math.abs(touchX-W/2) > Math.abs(touchY-H/2)) {
                    key = (touchX-W/2 < 0) ? KEY_LEFT:KEY_RIGHT;
                } else {
                    key = (touchY-H/2 < 0) ? KEY_UP:KEY_DOWN;
                }
            } else if (scene == S_APPEAR || scene == S_ATTACK ||
                    scene==S_DEFENCE || scene == S_ESCAPE) {
                key = KEY_SELECT;
            } else if (scene == S_COMMAND) {
                if (W/2-250 < touchX && touchX < W/2 &&
                    H-190 < touchY && touchY < H) {
                    key = KEY_1;
                } else if (W/2 < touchX && touchX < W/2+250 &&
                    H-190 < touchY && touchY < H) {
                    key = KEY_2;
                }
            }
        }
        return true;
    }

    //決定キー待ち
    private void waitSelect() {
        key = KEY_NONE;
        while (key != KEY_SELECT) sleep(100);
    }

    //スリープ
    private void sleep(int time) {
        try {
            Thread.sleep(time);
        } catch (Exception e) {
        }
    }

    //乱数の取得
    private static Random rand = new Random();
    public static int rand(int num) {
        return (rand.nextInt()>>>1)%num;
    }

    //ビットマップの読み込み
    private static Bitmap readBitmap(Context context, String name) {
        int resID = context.getResources().getIdentifier(
            name, "drawable", context.getPackageName());
        return BitmapFactory.decodeResource(
            context.getResources(), resID);
    }
}