package net.npaka.twitterclient;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import twitter4j.Status;
import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;
import twitter4j.auth.AccessToken;
import twitter4j.auth.OAuthAuthorization;
import twitter4j.auth.RequestToken;
import twitter4j.conf.Configuration;
import twitter4j.conf.ConfigurationContext;

//Twitterクライアント
public class TwitterClient extends Activity {
    //コンシューマーキーとコンシューマーシークレットの設定(1)
    private final static String 
        CONSUMER_KEY    = "Consumer keyをココに記述",
        CONSUMER_SECRET = "Consumer secretをココに記述",
        CALLBACK        = "mycallback://SignTwitter";

    //定数
    private final static int 
        WC = LinearLayout.LayoutParams.WRAP_CONTENT,
        MP = LinearLayout.LayoutParams.MATCH_PARENT,
        MENU_SETUP    = 0,
        MENU_UPDATE   = 1;
    private final static String
        BR = System.getProperty("line.separator");
    
    //システム
    private static Handler handler = new Handler();//ハンドラ
    private static float              dpi;        //DPI
    private static OAuthAuthorization oauth;      //OAuth認証
    private static RequestToken       req;        //リクエストトークン
    private static String             token;      //トークン
    private static String             tokenSecret;//トークンシークレット

    //UI
    private ListView listView;
    private Map<String, Bitmap> icons = new HashMap<String, Bitmap>();
    private List<Status> statuses = new ArrayList<Status>();

    //アクティビティ起動時に呼ばれる
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        
        //プログレスの有効化(2)
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        
        //DPIの取得(3)
        DisplayMetrics metrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(metrics); 
        dpi = metrics.density;
        
        //リストビューの生成
        listView = new ListView(this);
        listView.setLayoutParams(new LinearLayout.LayoutParams(MP, WC));
        listView.setAdapter(new ListAdapter());
        setContentView(listView);

        //トークンの読み込み
        SharedPreferences pref = getSharedPreferences("token", MODE_PRIVATE);
        token       = pref.getString("token", "");
        tokenSecret = pref.getString("tokenSecret", "");
    }

    //レジューム時に呼ばれる
    @Override
    protected void onResume() {
        super.onResume();

        //ブラウザのコールバックで起動した時の処理(5)
        final Uri uri = getIntent().getData();
        if (uri != null && uri.toString().startsWith(CALLBACK) &&
            uri.getQueryParameter("oauth_verifier") != null) {
            Thread thread = new Thread(new Runnable() {public void run() {
                try {
                    //Twitterのユーザー認証結果の取得
                    String verifier = uri.getQueryParameter("oauth_verifier");
                    AccessToken accToken = oauth.getOAuthAccessToken(req, verifier);
                    token = accToken.getToken();
                    tokenSecret = accToken.getTokenSecret();

                    //トークンの書き込み
                    SharedPreferences pref = getSharedPreferences("token", MODE_PRIVATE);
                    SharedPreferences.Editor editor = pref.edit();
                    editor.putString("token", token);
                    editor.putString("tokenSecret", tokenSecret);
                    editor.commit();

                    //ステータスの更新
                    handler.post(new Runnable() {public void run() {
                        updateStatuses();
                    }});
                } catch (TwitterException e) {
                    toast(TwitterClient.this, e.getMessage());
                }
            }});
            thread.start();
        }
        //ステータスの更新
        else {
            handler.post(new Runnable() {public void run() {
                updateStatuses();
            }});
        }
    }
    
    //オプションメニューの生成
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        MenuItem item0 = menu.add(0, MENU_SETUP, 0, "設定");
        item0.setIcon(android.R.drawable.ic_menu_preferences);
        MenuItem item1 = menu.add(0, MENU_UPDATE, 0, "更新");
        item1.setIcon(android.R.drawable.ic_menu_upload);
        return true;
    }    

    //メニューアイテム選択イベントの処理
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        
        //Twitterのユーザー認証
        if (id == MENU_SETUP) {
            startOAuth();
        }
        //ステータスの更新
        else if (id == MENU_UPDATE) {
            updateStatuses();
        }
        return true;
    }
    
    //Twitterのユーザー認証(4)
    private void startOAuth() {
        Thread thread = new Thread(new Runnable() {public void run() {
            //Oauth認証オブジェクトの生成
            Configuration conf = ConfigurationContext.getInstance();
            oauth = new OAuthAuthorization(conf);
            oauth.setOAuthConsumer(CONSUMER_KEY, CONSUMER_SECRET);

            //認証URLを取得
            try {
                req = oauth.getOAuthRequestToken(CALLBACK);
            } catch (TwitterException e) {
                toast(TwitterClient.this, e.getMessage());
            }
            String uri = req.getAuthorizationURL();

            //ブラウザの起動
            startActivity(new Intent(Intent.ACTION_VIEW , Uri.parse(uri)));
            finish();
        }});
        thread.start();
    }

    //ステータスの更新
    private void updateStatuses() {
        //Twitterのユーザー認証
        if (token.length() == 0) {
            startOAuth();
            return;
        }
        
        //不定プログレスの表示(2)
        setProgressBarIndeterminateVisibility(true);
        
        //ステータスの更新(6)
        Thread thread = new Thread(new Runnable() {public void run() {
            try {
                //ステータスの読み込み
                Twitter twitter = new TwitterFactory().getInstance();
                twitter.setOAuthConsumer(CONSUMER_KEY, CONSUMER_SECRET);
                twitter.setOAuthAccessToken(new AccessToken(token, tokenSecret));
                statuses = twitter.getHomeTimeline();
                icons.clear();
            
                handler.post(new Runnable() {public void run() {
                    //不定プログレスの非表示(2)
                    setProgressBarIndeterminateVisibility(false);

                    //リストビューの更新(8)
                    ((BaseAdapter) listView.getAdapter()).notifyDataSetChanged();
                }});
            } catch (TwitterException e) {
                toast(TwitterClient.this, e.getMessage());
            }
        }});
        thread.start();
    }
    
    //トーストの表示
    public static void toast(final Activity activity, final String text) {
        handler.post(new Runnable() {public void run() {
            //不定プログレスの非表示(2)
            activity.setProgressBarIndeterminateVisibility(false);
            
            //トーストの表示
            Toast.makeText(activity, text, Toast.LENGTH_LONG).show();
        }});
    }
    
    //リストアダプタの生成
    public class ListAdapter extends BaseAdapter {
        //項目数の取得
        @Override
        public int getCount() {
            return statuses.size();
        }

        //項目の取得
        @Override
        public Object getItem(int pos) {
            return statuses.get(pos);
        }

        //項目IDの取得
        @Override
        public long getItemId(int pos) {
            return pos;
        }

        //ビューの取得
        @Override
        public View getView(int pos, View view, ViewGroup parent) {
            Context context = TwitterClient.this;
            Status status = statuses.get(pos);
                
            //レイアウトの生成
            if (view == null) {
                int padding = (int)(6*dpi);
                LinearLayout layout = new LinearLayout(context);
                layout.setBackgroundColor(Color.WHITE);
                layout.setPadding(padding, padding, padding, padding);
                layout.setGravity(Gravity.TOP);
                view = layout;

                //アイコン
                int size = (int)(48*dpi);
                ImageView imageView = new ImageView(context);
                imageView.setTag("icon");
                imageView.setLayoutParams(new LinearLayout.LayoutParams(size, size));
                layout.addView(imageView);
                
                //テキストの指定
                TextView textView = new TextView(context);
                textView.setTextColor(Color.BLACK);
                textView.setTag("text");
                textView.setPadding(padding, 0, padding, 0);
                layout.addView(textView);
            }
                
            //ステータスの値の取得(7)
            ImageView imageView = (ImageView)view.findViewWithTag("icon");
            readIcon(imageView, status.getUser().getBiggerProfileImageURL().toString());
            TextView textView = (TextView)view.findViewWithTag("text");
            textView.setText("【"+status.getUser().getName()+"】"+BR+
                status.getText());
            return view;
        }        
    }
    
    //アイコンの読み込み
    private void readIcon(final ImageView imageView, final String url) {
        //キャッシュ内アイコンの取得
        if (icons.containsKey(url)) {
            imageView.setImageBitmap(icons.get(url));
            return;
        }
        
        //ネット上アイコンの取得
        imageView.setImageBitmap(null);
        Thread thread = new Thread(new Runnable() {public void run() {
            try {
                InputStream input = (new URL(url)).openStream();
                final Bitmap icon = BitmapFactory.decodeStream(input);
                if (icon != null) {
                    icons.put(url, icon);
                    handler.post(new Runnable() {public void run() {
                        imageView.setImageBitmap(icon);
                    }});
                }
            } catch (Exception e) {
                toast(TwitterClient.this, e.getMessage());
            }
        }});
        thread.start();
    }
}